# Generated by roxygen2: do not edit by hand

S3method(print,summary.GSCIMC)
S3method(print,summary.SARARgamlss)
export(GSCIMC)
export(Hurdle_GSCIMC)
export(SARARgamlss)
export(ptfamily)
export(summary_SAR)
export(var_rho_inv)
import(gamlss)
import(splines)
importFrom(gamlss.dist,NO)
importFrom(methods,is)
importFrom(sphet,spreg)
importFrom(stats,.getXlevels)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,dpois)
importFrom(stats,fitted)
importFrom(stats,gaussian)
importFrom(stats,glm.fit)
importFrom(stats,make.link)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,model.response)
importFrom(stats,optim)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,rpois)
importFrom(stats,update)
