# spatemR
An R package for autoregresive spatial models with autoregresive temporal trend and GAMLSS

Use `devtools::install_github("Cruzalirio/spatemR")` to install the latest version
of the package
